@@ -8,8 +8,10 @@ typedef struct _gaainfo gaainfo;
 
 struct _gaainfo
 {
-#line 90 "certtool.gaa"
+#line 94 "certtool.gaa"
 	int debug;
+#line 90 "certtool.gaa"
+	char *template;
 #line 87 "certtool.gaa"
 	char *infile;
 #line 84 "certtool.gaa"
@@ -53,7 +55,7 @@ extern "C"
 
     void gaa_help(void);
     
-    int gaa_file(char *name, gaainfo *gaaval);
+    int gaa_file(const char *name, gaainfo *gaaval);
     
 #ifdef __cplusplus
 }
